==> Lingy-0.1.3.tar.gz: Lingy-0.1.3/lib/Lingy/Printer.pm <==
use strict; use warnings;
package Lingy::Printer;

use Lingy::Common;

use Scalar::Util 'blessed';
use Sub::Identify 'sub_name';

sub new { bless {}, shift }

my $escape = {
    "\n" => "\\n",
    "\t" => "\\t",
    "\"" => "\\\"",
    "\\" => "\\\\",
};

sub pr_str {
    my ($o, $raw) = (@_, 0);
    my $type = ref $o;

    # Hack to allow map key strings to print like symbols:
    if (not $type and $o =~ /$symbol_re$/) {
        $type = 'Lingy::Lang::KeySymbol';
    }

    $type or XXX $o, "Don't know how to print internal value '$o'";

    $type eq 'Lingy::Lang::Atom' ? "(atom ${\pr_str($o->[0], $raw)})" :
    $type eq 'Lingy::Lang::String' ? $raw ? $$o :
        qq{"${local $_ = $$o; s/([\n\t\"\\])/$escape->{$1}/ge; \$_}"} :
    $type eq 'Lingy::Lang::Regex' ? $raw ? $$o :
        qq{#"${local $_ = $$o; s/([\n\t\"\\])/$escape->{$1}/ge; \$_}"} :
    $type eq 'Lingy::Lang::KeySymbol' ? $o :
    $type eq 'Lingy::Lang::Symbol' ? $$o :
    $type eq 'Lingy::Lang::Keyword' ? $$o :
    $type eq 'Lingy::Lang::Number' ? $$o :
    $type eq 'Lingy::Lang::Boolean' ? $$o ? 'true' : 'false' :
    $type eq 'Lingy::Lang::Nil' ? 'nil' :
    $type eq 'Lingy::Lang::Var' ? ("#'" . $$o) :
    $type eq 'Lingy::Lang::Class' ? $o->_name :
    $type eq 'Lingy::Lang::Character' ? $o->print($raw) :
    $type eq 'Lingy::Lang::Type' ? $$o :
    $type eq 'CODE' ? "#<function ${\ sub_name($o)}>" :
    $type eq 'Lingy::Lang::Function' ? '#<Function>' :
    $type eq 'Lingy::Lang::Macro' ? '#<Macro>' :
    $type eq 'Lingy::Lang::List' ?
        "(${\ join(' ', map pr_str($_, $raw), @$o)})" :
    $type eq 'Lingy::Lang::Vector' ?
        "[${\ join(' ', map pr_str($_, $raw), @$o)}]" :
    $type eq 'Lingy::Lang::HashMap' ?
        "{${\ join(', ', map {
            my ($key, $val) = ($_, $o->{$_});
            if ($key =~ /^:/) {
                $key = keyword($key);
            } elsif ($key =~ s/^\"//) {
                $key = string($key);
            } elsif ($key =~ s/^(\S+) $/$1/) {
                $key = symbol($key);
            }
            (pr_str($key, $raw) . ' ' . pr_str($val, $raw))
        } keys %$o)}}" :
    $type =~ /^(?:(?:quasi|(?:splice_)?un)?quote|deref)$/ ?
        "(${$type=~s/_/-/g;\$type} ${\ pr_str($o->[0], $raw)})" :
    $type eq 'Lingy::Env' ? '#<Env>' :
    $type eq 'lingy-internal' ? "" :
    not(blessed($o)) ? do {
        WWW($o);
        err "Tried to print the unblessed internal reference above";
    } :
    $o->isa('Lingy::Namespace') ? qq(#<Namespace ${\ $o->NAME}>) :
    die "Don't know how to pr_str: '$o'";
}

1;

==> Lingy-0.1.5.tar.gz: Lingy-0.1.5/lib/Lingy/Printer.pm <==
use strict; use warnings;
package Lingy::Printer;

use Lingy::Common;

use Scalar::Util 'blessed';
use Sub::Identify 'sub_name';

sub new { bless {}, shift }

my $escape = {
    "\n" => "\\n",
    "\t" => "\\t",
    "\"" => "\\\"",
    "\\" => "\\\\",
};

sub pr_str {
    my ($o, $raw) = (@_, 0);
    my $type = ref $o;

    # Hack to allow map key strings to print like symbols:
    if (not $type and $o =~ /$symbol_re$/) {
        $type = 'Lingy::Lang::KeySymbol';
    }

    $type or XXX $o, "Don't know how to print internal value '$o'";

    $type eq ATOM ? "(atom ${\pr_str($o->[0], $raw)})" :
    $type eq STRING ? $raw ? $$o :
        qq{"${local $_ = $$o; s/([\n\t\"\\])/$escape->{$1}/ge; \$_}"} :
    $type eq REGEX ? $raw ? $$o :
        qq{#"${local $_ = $$o; s/([\n\t\"\\])/$escape->{$1}/ge; \$_}"} :
    $type eq 'Lingy::Lang::KeySymbol' ? $o :
    $type eq SYMBOL ? $$o :
    $type eq KEYWORD ? $$o :
    $type eq NUMBER ? $$o :
    $type eq BOOLEAN ? $$o ? 'true' : 'false' :
    $type eq NIL ? 'nil' :
    $type eq VAR ? ("#'" . $$o) :
    $type eq CLASS ? $o->_name :
    $type eq CHARACTER ? $o->print($raw) :
    $type eq 'CODE' ? "#<function ${\ sub_name($o)}>" :
    $type eq FUNCTION ? '#<Function>' :
    $type eq MACRO ? '#<Macro>' :
    $type eq LIST ?
        "(${\ join(' ', map pr_str($_, $raw), @$o)})" :
    $type eq VECTOR ?
        "[${\ join(' ', map pr_str($_, $raw), @$o)}]" :
    $type eq HASHMAP ?
        "{${\ join(', ', map {
            my ($key, $val) = ($_, $o->{$_});
            if ($key =~ /^:/) {
                $key = keyword($key);
            } elsif ($key =~ s/^\"//) {
                $key = string($key);
            } elsif ($key =~ s/^(\S+) $/$1/) {
                $key = symbol($key);
            }
            (pr_str($key, $raw) . ' ' . pr_str($val, $raw))
        } keys %$o)}}" :
    $type =~ /^(?:(?:quasi|(?:splice_)?un)?quote|deref)$/ ?
        "(${$type=~s/_/-/g;\$type} ${\ pr_str($o->[0], $raw)})" :
    $type eq 'Lingy::Env' ? '#<Env>' :
    $type eq 'lingy-internal' ? "" :
    not(blessed($o)) ? do {
        WWW($o);
        err "Tried to print the unblessed internal reference above";
    } :
    $o->isa('Lingy::Namespace') ? qq(#<Namespace ${\ $o->NAME}>) :
    die "Don't know how to pr_str: '$o'";
}

1;
